/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.collision;

import java.util.ArrayList;
import java.util.Collection;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the results of a {@link FloodDurationCalculation}.
 *
 * @author Gernot Belger
 */
final class CollisionCalcDetailResult extends AbstractCalculationExportableResult<CollisionCalculationResults> {

    private static final long serialVersionUID = 1L;
    private static final String JASPER_FILE = "/jasper/templates/sinfo.collision.detail.jrxml";

    public CollisionCalcDetailResult(final String label, final Collection<ResultRow> rows) {
        super(label, rows);
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results) {

        exportContextCSV.writeBlankLine();
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final RiverInfo river) {

        final int colSize = 6;
        exportContextCSV.writeTitleForTabs("sinfo.export.csv.title.collision.detail", 6); // Voraussetzung für Tabs ist, dass der Titel vor den Headern
                                                                                          // geschrieben wird.
        // Das ist etwas doof.

        final Collection<String> header = new ArrayList<>(colSize);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.date));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.collisionGaugeW));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.gaugeLabel));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.discharge));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.dischargeZone));

        exportContextCSV.writeCSVLine(header.toArray(new String[colSize]));
    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {
        return formatRow(exportContextCSV, results, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {
        /*
         * final Collection<String> lines = new ArrayList<>(6);
         * lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
         * return lines.toArray(new String[lines.size()]);
         */
        return formatRow(exportContextCSV, results, row);
    }

    private String[] formatRow(final ExportContextCSV exportContextCSV, final CollisionCalculationResults results, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(6);

        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.date));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.collisionGaugeW));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.gaugeLabel));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.discharge));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.dischargeZone));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    protected String getJasperFile() {
        return JASPER_FILE;
    }

    @Override
    protected void addJRTableHeader(final ExportContextCSV exportContextCSV, final MetaAndTableJRDataSource source, final CollisionCalculationResults results) {

        /* column headings */
        exportContextCSV.addJRMetadata(source, "station_header", GeneralResultType.station);
        exportContextCSV.addJRMetadata(source, "date_header", GeneralResultType.date);
        exportContextCSV.addJRMetadata(source, "gaugew_header", SInfoResultType.collisionGaugeW);
        exportContextCSV.addJRMetadata(source, "gaugelabel_header", SInfoResultType.gaugeLabel);
        exportContextCSV.addJRMetadata(source, "discharge_header", SInfoResultType.discharge);
        exportContextCSV.addJRMetadata(source, "dischargezone_header", SInfoResultType.dischargeZone);
    }
}